@@ -200,6 +200,24 @@ def _set_conn_read_only(conn):
         print(f"couldn't set read-only mode on connection: {conn.__class__.__name__}")
 
 
+def enquote_sql_identifier(name, allow_separator: bool=True, flavor: str=None):
+    name = str(name)
+    # '.' in name will be treated as separator between schema and table name
+    if "." in name and allow_separator:
+        return ".".join(map(enquote_sql_identifier, name.split('.')))
+    # postgres needs quoting of uppercase
+    if flavor == "postgres":
+        if re.search(r'[^a-z0-9_]', name) or name[:1].isdigit():
+            return f'"{name}"'
+        return name
+    # all other engines
+    if re.search(r'[^A-Za-z0-9_]', name) or name[:1].isdigit():
+        if flavor == "mysql":
+            return f'`{name}`'
+        return f'"{name}"'
+    return name
+
+
 def dataframe_to_sql(dataframe, table_name: str, append: bool=False, flavor: str=None):
     """
     Generate SQL statements to create a table and add data.
@@ -209,26 +227,26 @@ def dataframe_to_sql(dataframe, table_name: str, append: bool=False, flavor: str
     :param flavor:      Name of a particular type of database, to adjust details of SQL format.
     :returns a generator of (sql, parameter list)
     """
-    def enquote(name):
-        name = str(name)
-        if re.search(r'[^A-Za-z0-9_]', name) or name[:1].isdigit():
-            return f'"{name}"'
-        return name
+    enquote = lambda name, **k: enquote_sql_identifier(name, flavor=flavor, **k)
     if not append:
         yield "DROP TABLE IF EXISTS %s" % enquote(table_name), []
+    # TODO expand to cover more types
     if flavor == "bigquery":
-        dtype_to_sql = {"int64": "INT64", "float64": "NUMERIC"}
+        dtype_to_sql = {"int32": "INT64", "int64": "INT64", "float64": "NUMERIC", "datetime64": "TIMESTAMP"}
         str_type = "STRING"
     else:
-        dtype_to_sql = {"int64": "INTEGER", "float64": "DOUBLE PRECISION"}
+        dtype_to_sql = {"int32": "INTEGER", "int64": "INTEGER", "float64": "DOUBLE PRECISION", "datetime64": "TIMESTAMP"}
         str_type = "VARCHAR"
     cols = list(dataframe.columns)
     types = []
     for col, dtype in zip(cols, dataframe.dtypes):
-        sql_type = dtype_to_sql.get(str(dtype)) or f"{str_type}({dataframe[col].str.len().max()+3:d})"
+        sql_type = dtype_to_sql.get(str(dtype))
+        if not sql_type:
+            max_len = int(dataframe[col].str.len().max()) + 3
+            sql_type = f"{str_type}({max_len:d})"
         types.append(sql_type)
     rows = dataframe.itertuples(index=False, name=None)
-    fields = ", ".join("%s %s" % (enquote(col), t) for col, t in zip(cols, types))
+    fields = ", ".join("%s %s" % (enquote(col, allow_separator=False), t) for col, t in zip(cols, types))
     yield "CREATE TABLE IF NOT EXISTS %s (%s)" % (enquote(table_name), fields), []
     ins0 = "INSERT INTO %s (%s) VALUES " % (enquote(table_name), ", ".join(map(enquote, cols)))
     sqls = []
